require "defines"
require "util"

find_text_output={}
remote.add_interface("signposts",
    {find_text=function() 
        local res={}
        for _,p in pairs(game.players) do 
            local x=p.position.x
            local y=p.position.y 
            local r=p.surface.find_entities_filtered{area={{x-10,y-10},{x+10,y+10}},name="flying-text"}
            if #r>0 then for _,v in pairs(r) do table.insert(res,v) end end 
        end
        for _,p in pairs(game.players) do
            if #res>0 then
                find_text_output=res
                p.print({"interface-found"})
                for k,v in pairs(res) do
                    p.print(""..k.." : x="..v.position.x..", y="..v.position.y)
                end
                p.print({"interface-to-remove"})
            else p.print({"interface-found-nothing"})
            end
        end
        end,
    remove_text=function(i)
            i=tonumber(i)
            if find_text_output then
                if find_text_output[i] then
                    find_text_output[i].destroy()
                else
                    for _,p in pairs(game.players) do
                        p.print({"interface-no-such-result"})
                    end
                end
            else
                for _,p in pairs(game.players) do
                    p.print({"interface-no-results"})
                end
            end
        end
    })

function init()
    --classes
    post={
        new=function(post_class,entity)
            local sign={
                head="",--always shown string 
                body={""},--strings shown in gui only
                entity=entity,--post entity
                ent={pos=entity.position,force=entity.force,surf=entity.surface};--backup
                text=nil--link to flytext entity
                }
            entity.destructible=false
            setmetatable(sign,{__index=post_class})
            return sign
            end,
        rebuild=function(sign)
            local s=sign.ent
            local e=s.surf.create_entity{name="sign-post",position=s.pos,force=s.force}
            if sign.entity and sign.entity.valid then sign.entity.destroy() end
            sign.entity=e.valid and e
            end,
        retext=function(sign)
            if sign.head and sign.head~="" then                
                local s=sign.ent
                e=s.surf.create_entity{
                    name='flying-text',
                    position={s.pos.x-1,s.pos.y-1},
                    force=s.force,
                    text=sign.head,}
                e.active=false
                if sign.text and sign.text.valid then sign.text.destroy() end
                sign.text=e
            end
            end,
        hed=function(sign,strin)
            if strin then sign.head=strin end
            end,
        bod=function(sign,strins)
            if strins then
                sign.body={}
                for _,strin in pairs(strins) do
                    table.insert(sign.body,strin)
                end
            end
            end,
        destroy=function(sign)
            if sign.entity and sign.entity.valid then sign.entity.destroy() end
            if sign.text and sign.text.valid then sign.text.destroy() end
            end
    }
    --globals
    sign_list={}
    function sign_list:put(s) table.insert(self,s) end
    function sign_list:find(entity)
        local n=1
        while (n<=#self) and self[n].entity~=entity do
            n=n+1
        end
        return (n<=#self) and n
        end
    function sign_list:rem(n) table.remove(self,n) end
    opened={
        put=function(self,playerind,sign) self[playerind]=sign end,
        rem=function(self,playerind) self[playerind]=nil end,
    }
    --make definitions stored
    global.post=post
    global.sign_list=sign_list
    global.opened=opened
end
script.on_init(init)
script.on_load(function()
    post = global.post
    sign_list = global.sign_list
    opened = global.opened
    for _,p in ipairs(sign_list) do setmetatable(p,{__index=post}) end
end)
    

--event handlers
script.on_event(defines.events.on_built_entity,
    function(event)
        if event.created_entity.name=='sign-post' then
            local sign=post:new(event.created_entity)
            sign_list:put(sign)
            opened:put(event.player_index,sign)
            draw_gui(event.player_index,sign)
        elseif event.created_entity.name=='sign-post-map-dummy' then
            local e=event.created_entity
            local p=game.get_player(event.player_index)
            local m=e.surface.create_entity{name='sign-post-map-mark', position=e.position, force=e.force}
            m.destructible=false
            e.destroy()
        end
    end)
script.on_event(defines.events.on_preplayer_mined_item,
    function(event)
        if event.entity.name=='sign-post' then
            local n=sign_list:find(event.entity)
            if not n then 
                sign_list:put(post:new(event.created_entity))
                n=#sign_list
            end
            sign_list[n]:rebuild()
            opened:put(event.player_index,sign_list[n])
            draw_gui(event.player_index,sign_list[n])
        end
    end)
script.on_event(defines.events.on_marked_for_deconstruction, 
    function(event)
        if event.entity.name=='sign-post' or event.entity.name=='sign-post-map-mark' then
            event.entity.cancel_deconstruction(event.entity.force)
        end
    end)
--all about gui
--gui drawers. i suck at guing
function draw_gui(player_index,sign)--[[Main sign post gui]]
    if game.players[player_index].gui.center.SignPosts then
        game.players[player_index].gui.center.SignPosts.destroy()
    end
    game.players[player_index].gui.center.add{type="frame", name="SignPosts", caption={"gui-main-caption"}, direction="vertical"}
    local root=game.players[player_index].gui.center.SignPosts
    root.add{type='textfield',name='head'}
    root.head.text=sign.head
    root.add{type='frame',name="body",direction="vertical",caption={"gui-text"}}
    root.body.add{type="flow", name="lines",direction="vertical"}
    local i=1
    while i<=(#sign.body) do 
        root.body.lines.add{type='textfield',name=tostring(i)}
        root.body.lines[tostring(i)].text=sign.body[i]
        i=i+1
    end
    root.body.add{type="flow", name="buttons",direction="horizontal"}
    root.body.buttons.add{type='button',name="lineadd",caption={"gui-button-lineadd"}}
    root.body.buttons.add{type='button',name="linerem",caption={"gui-button-linerem"}}
    
    root.add{type="flow", name="buttons",direction="horizontal"}
    root.buttons.add{type="button", name="spwrite", caption={"gui-button-write"}}
    root.buttons.add{type="button", name="spcancel", caption={"gui-button-cancel"}}
    root.buttons.add{type="button", name="spdestroy", caption={"gui-button-destroy"}}
    
    
end
function draw_confirm(player_index)
    if game.players[player_index].gui.center.SignPostsConfirm then
        game.players[player_index].gui.center.SignPostsConfirm.destroy()
    end
    game.players[player_index].gui.center.add{type="frame", name="SignPostsConfirm", caption={"gui-confirm-caption"}, direction="vertical"}
    local root=game.players[player_index].gui.center.SignPostsConfirm
    root.add{type="button", name="yes", caption={"gui-button-confirm-yes"}}
    root.add{type="button", name="no", caption={"gui-button-confirm-no"}}
end
--button event
script.on_event(defines.events.on_gui_click,
    function(event)
        if ( game.get_player(event.player_index).gui.center.SignPosts~=nil) then
            local mygui=game.get_player(event.player_index).gui.center.SignPosts
            if event.element == mygui.buttons.spwrite then
                spwrite(event.player_index,mygui)
            elseif event.element == mygui.buttons.spcancel then
                spcancel(event.player_index,mygui)
            elseif event.element == mygui.buttons.spdestroy then
                draw_confirm(event.player_index)
            elseif event.element == mygui.body.buttons.lineadd then
                addline(mygui.body.lines)
            elseif event.element == mygui.body.buttons.linerem then
                remline(mygui.body.lines)
            end
        end
        if ( game.get_player(event.player_index).gui.center.SignPostsConfirm~=nil) then
            local confgui=game.get_player(event.player_index).gui.center.SignPostsConfirm
            if event.element==confgui.yes then
                spdestroy(event.player_index)
            elseif event.element==confgui.no then
                confgui.destroy()
            end
        end
    end)
--gui button handling
function addline(lines)
    lines.add{type='textfield',name=tostring(#lines.children_names+1)}
end
function remline(lines)
    if #lines.children_names>1 then
        lines[tostring(#lines.children_names)].destroy()
    end
end
function spwrite(index,gui)
    local sign=opened[index]
    sign:hed(gui.head.text)
    lines={}
    for _,l in pairs(gui.body.lines.children_names) do
        table.insert(lines,gui.body.lines[l].text) 
    end
    sign:bod(lines)
    sign:retext()
    opened:rem(index)
    gui.destroy()
end
function spcancel(index,gui)
    --this function exists only in case more complicated cancell behavior will be needed later
    opened:rem(index)
    gui.destroy()
end
function spdestroy(index)
    local sign=opened[index]
    sign_list:rem(sign_list:find(sign.entity))
    if sign.entity.valid then
        sign:destroy()
        game.get_player(index).insert{name="sign-post",count=1};
    end
    opened:rem(index)
    game.get_player(index).gui.center.SignPostsConfirm.destroy()
    game.get_player(index).gui.center.SignPosts.destroy()
end

script.on_configuration_changed(
function(data)
    if data.mod_changes ~= nil and data.mod_changes["SignPosts"] ~= nil and data.mod_changes["SignPosts"].old_version == "0.2.0" then
        init()
        for _,p in pairs(game.players) do 
            if p.gui.center.SignPosts then p.gui.center.SignPosts.destroy() end
        end
        for _,f in pairs(game.forces) do
            f.reset_technologies()
            f.reset_recipes()
        end
        for _,s in pairs(global.signs) do
            local n=post:new(s.sign)
            n:hed(s.str)
            n.text=s.text
            sign_list:put(n)
        end
        global.signs=nil
        global.last_built=nil
        global.last_pos=nil
        global.last_str=nil
        for _,p in pairs(game.players) do 
            p.print("Migration done")
        end
    end
end)
